#! /usr/bin/env node

const argv = require('minimist')(process.argv.slice(2))

const api = require('../lib/api.js')

const debug = require('@ff0000-ad-tech/debug')
var log = debug('build-source-automator')

/**
 *  --context, path to the ad-folder
 *  --size, when compiling traffic, defaults to 300x250
 * 
 *	--wpCsPath, path to wp-creative-server
 *		default assumes it was installed at the context of each request
 *
 *	--wpDmPath, path to wp-deploy-manager
 * 		default assumes it was installed in node_modules, next to wp-creative-server
 *
 */
const config = {
	cwd: argv.cwd,
	wpCsPath: argv.wpCsPath || './node_modules/@ff0000-ad-tech/wp-creative-server',
	wpDmPath: argv.wpDmPath || './node_modules/@ff0000-ad-tech/wp-deploy-manager'
}

// default API command
let cmd = argv.cmd
if (!cmd) {
	cmd = 'batch-compile'
}

// verbose
const verbose = argv.verbose && !!argv.verbose.match(/true/)

// non-zero exits indicate process errored
function exit(err) {
	if (err) {
		console.error(err)
		process.exit(1)
	} else {
		process.exit(0)
	}
}
function returnCsInfo(cs) {
	// output Creative Server details
	if (cs) {
		process.stdin.resume() // keep the process running, to return subprocess output
		console.log('CREATIVE SERVER')
		console.log(JSON.stringify(cs, null, 2))
	}
}

// node ./bin/build-source-automator.js --cmd "compile-traffic" --context "/Users/greg.connell/Documents/_CLIENTS/_AD_APP_TESTING/RED_Dev/ES6/standard-base" --verbose
switch (cmd) {
	// INSTALL TOP
	case 'install-top':
		if (!argv.context) {
			log(`--context is required. It should point at a FAT Build Source.`)
		}
		api
			.installTop(argv.context, verbose)
			.then(() => exit())
			.catch(err => exit(err))
		break

	// INSTALL BUILD
	case 'install-build':
		if (!argv.context) {
			log(`--context is required. It should point at a FAT Build Source.`)
		}
		api
			.installBuild(argv.context, verbose)
			.then(() => exit())
			.catch(err => exit(err))
		break

	// LAUNCH CS APP
	case 'launch-cs-app':
		if (!argv.context) {
			log(`--context is required. It should point at a FAT Build Source.`)
		}
		api
			.launchCsApp(config, argv.context, verbose)
			.then(cs => returnCsInfo(cs))
			.catch(err => exit(err))
		break

	// COMPILE TRAFFIC
	case 'compile-traffic':
		if (!argv.context) {
			log(`--context is required. It should point at a FAT Build Source.`)
		}
		api
			.compileTraffic(config, argv.context, argv.size || '300x250', verbose)
			.then(() => exit())
			.catch(err => exit(err))
		break
}
