const fs = require('fs')
const spawn = require('child_process').spawn
const cmdEscape = require('@ff0000-ad-tech/cmd-escape')

const paths = require('./paths.js')

const log = require('@ff0000-ad-tech/debug')('build-source-automator:npm-manager')

function updateDeps(path, dependencies) {
	log('Updating NPM dependencies:\n')
	return new Promise((resolve, reject) => {
		// dependency promises (each requested dependency)
		let promises = []
		;(dependencies || []).forEach(dependency => {
			/* {
					component: [github org/name "full_name"],
					repo: [github "ssh_url"],
					branch: [github "ref"],
					pkg: ["top" or "build"]
				}
			} */
			;(dep => {
				log(` - ${dep.component}`)
				promises.push(
					(() => {
						const pkgPath = dep.pkg === 'build' ? paths.BUILD_PKG_PATH() : paths.TOP_PKG_PATH()
						return npmManager.updatePkgDep(`${path}/${pkgPath}`, `@${dep.component}`, dep.repo, dep.branch)
					})()
				)
			})(dependency)
		})
		Promise.all(promises)
			.then(() => resolve())
			.catch(err => reject(err))
	})
}

// load a "package.json" at path, update dependency with ssh-url#branch
function updatePkgDep(path, dependency, sshUrl, prBranch) {
	return new Promise((resolve, reject) => {
		log(`Pointing Test-Container's package.json at PR-Component's branch:`)
		log(` ${dependency} > ${sshUrl}#${prBranch}`)
		if (!fs.existsSync(path)) {
			return reject(new Error(`Test Container is missing "package.json" at ${path}`))
		}
		fs.readFile(path, (err, data) => {
			if (err) {
				return reject(err)
			}
			let pkg = JSON.parse(data)
			pkg.dependencies[dependency] = `git+ssh://${sshUrl}#${prBranch}`
			fs.writeFile(path, JSON.stringify(pkg, null, 2), err => {
				if (err) {
					return reject(err)
				}
				resolve()
			})
		})
	})
}

// run npm install on a container
function install(path, verbose) {
	return new Promise((resolve, reject) => {
		log(`NPM installing ${path}`)
		let output = ''
		const cmd = 'npm'
		const cmdArgs = ['install']
		const args = []
		const env = Object.create(process.env)
		env.NODE_ENV = 'development'
		log(` CLI: cd "${path}" && ${cmd} ${cmdArgs.join(' ')} ${cmdEscape(args)}`)
		const prc = spawn('npm', cmdArgs, { cwd: path, env })
		prc.stdout.on('data', data => {
			const stdout = data.toString('utf8')
			if (verbose) {
				log(stdout)
			}
			output += stdout
		})
		prc.stderr.on('data', data => {
			const stderr = data.toString('utf8')
			if (verbose) {
				log(stderr)
			}
			output += stderr
		})
		prc.on('close', code => {
			if (code === 0) {
				log(`  - install complete`)
				resolve()
			} else {
				reject(new Error(output))
			}
		})
	})
}

module.exports = {
	updateDeps,
	updatePkgDep,
	install
}
